<?php

namespace App\Classes\Models\Package;
use App\Classes\Models\BaseModel;

class Package extends BaseModel
{
    protected $table = 'package';
    protected $primaryKey = 'id';

    protected $entity = 'packages';
    protected $searchableColumns = ['plan_name','plan_description','monthly_price','yearly_price'];
    protected $fillable = ['id','plan_name','plan_description','monthly_price','yearly_price','note'];
    protected $per_page = '10';

    public function __construct( array $attributes = [] )
    {
        $this->bootIfNotBooted();
        $this->syncOriginal();
        $this->fill( $attributes );
        
    }

    public function list( $search = '', $page = 0, $sortedBy = ['id'], $sortedOrder = ['ASC'] )
    {
        $per_page = $this->per_page;
        $list = $this->setSelect()
                     ->addSearch( $search )
                     ->addOrderBy( $sortedBy, $sortedOrder )
                     ->addPaging( $page, $per_page )
                     ->addGroupBy( $this->table . '.id' )
                     ->get();

        return $list;
    }

    public function alllist( $search = '', $page = 0, $sortedBy = ['id'], $sortedOrder = ['ASC'] )
    {
        $per_page = $this->per_page;
        $list = $this->setSelect()
                     
                     ->addOrderBy( $sortedBy, $sortedOrder )
                     
                     ->addGroupBy( $this->table . '.id' )
                     ->get();

        return $list;
    }



    public function addGroupBy( $groupByName = null )
    {
        if ( ! empty( $groupByName ) ) {

            $this->queryBuilder->groupBy( $groupByName );
        }
        return $this;
    }

    public function addOrderBy( $sortedBy = [], $sortedOrder = [] )
    {

        if ( ! empty( $sortedBy ) && ! is_array( $sortedBy ) ) {
            $sortedBy = [$sortedBy];
        }

        if ( ! empty( $sortedOrder ) && ! is_array( $sortedOrder ) ) {
            $sortedOrder = [$sortedOrder];
        }

        if ( ! empty( $sortedBy ) && ! empty( $sortedOrder ) ) {

            foreach ( $sortedBy as $key => $sortedByName ) {

                $sortedOrder = ! empty( $sortedOrder[$key] ) ? $sortedOrder[$key] : $sortedOrder[0];

                $this->queryBuilder->orderBy( $sortedByName, $sortedOrder );
            }
        }
        return $this;
    }

    public function listTotalCount( $search = '', $sortedBy = ['id'], $sortedOrder = ['DESC'] )
    {
        $this->reset();
        $count = $this->setSelect()
                      ->addSearch( $search )
                      ->addOrderBy( $sortedBy, $sortedOrder )
                      ->addGroupBy( $this->table . '.id' )
                      ->get()
                      ->count();

        return $count;
    }

    public function preparePagination( $total, $basePath )
    {
        $perpage = $this->per_page;
        $pageHelper = new \App\Classes\Helpers\PageHelper( $perpage, 'page' );
        $pageHelper->set_total( $total );
        $pageHelper->page_links( $basePath );
        return $pageHelper->page_links( $basePath );
    }

    public function saveRecord( $data )
    {
        $rules = ['plan_name' => 'required|max:100','plan_description' => 'required|max:100', 'monthly_price'  => 'required','yearly_price'  => 'required'];

        if ( isset( $data['id'] ) && $data['id'] != '' ) 
        {
            $id = $data['id'];
        } 

        $validationResult = $this->validateData( $rules, $data );
        $result = [];
        $result['id'] = '';
        
        if ( $validationResult['success'] == false ) 
        {
            $result['success'] = false;
            $result['message'] = $validationResult['message'];
            $result['id'] = $data['id'];
            return $result;
        }

        // $data['status'] = 'active';

        $this->beforeSave( $data );
        if ( isset( $data['id'] ) && $data['id'] != '' ) 
        {
            $package = \App\Classes\Models\Package\Package::findOrFail( $data['id'] );
            $package->update( $data );
            $this->afterSave( $data, $package );
            $result['id'] = $package->id;
            $result['message'] = "プランを正常に更新しました。";

        } else {
            $package = \App\Classes\Models\Package\Package::create( $data );
            $result['id'] = $package->id;
            $this->afterSave( $data, $package );
            $result['message'] = "プランを正常に追加しました。";
        }

        $result['success'] = true;
        return $result;
    }

    public function removed( $id )
    {
        $this->beforeRemoved( $id );
        $deleteMemberObj = $this->display( $id );
        if ( ! empty( $deleteMemberObj ) ) {
            $delete = $deleteMemberObj->delete();
            $this->afterRemoved( $id );
            return $delete;
        }
        return false;
    }

    public function display( $id )
    {
        return $this->load( $id );
    }

    public function load( $id )
    {
        $this->beforeLoad( $id );
        $return = $this->setSelect()
                       ->addPackageIdFilter( $id )
                       ->get()
                       ->first();

        $this->afterLoad( $id, $return );
        return $return;
    }

    public function addPackageIdFilter( $id = 0 )
    {
        $this->queryBuilder->where( 'id', $id );
        return $this;
    }

    public function addStatusFilter( $status = 0 )
    {
        if($status > 0) {
            $this->queryBuilder->where( 'status', $status );
        }
        return $this;
    }

    public function getUserWidget()
    {
        return $this->setSelect()
                    ->get()
                    ->count();
    }

    public function getAdminList($status = 1)
    {
        return $this->setSelect()
                    ->addStatusFilter($status)
                     ->get();
    }

    public function expire_package_date($package_id)
    {
        $package            = $this->setSelect()->where('id', $package_id)->get();
        $start_date         = $package[0]->created_at->toDateTimeString();
        $expires_no_month   = $package[0]->expires_no_month;
        $expire_date        = date('Y-m-d H:i:s',strtotime($start_date.' +'.$expires_no_month.' month'));
        
        return $expire_date;
        // dd($package_id, $expires_no_month, $start_date, $expire_date);
    }

    public function statusupdate($package_id)
    {
        $data['status'] = 'expire';
        $package = \App\Classes\Models\Package\Package::findOrFail($package_id);
        $package->update( $data );
        $this->afterSave( $data, $package );
        // $result['id'] = $package->id;
    }

}