<?php

namespace App\Classes\Models\Subscription;
use App\Classes\Models\BaseModel;

class Subscription extends BaseModel
{
    protected $table = 'subscription';
    protected $primaryKey = 'id';

    protected $entity = 'subscriptions';
    protected $searchableColumns = ['subscription_id','user_id','status'];
    protected $fillable = ['id','user_id','package_id','interval','customer_id','product_id','price_id','subscription_id','current_period_start','current_period_end','canceled_at','status','holder_name' ];
    protected $per_page = '10';

    public function __construct( array $attributes = [] )
    {
        $this->bootIfNotBooted();
        $this->syncOriginal();
        $this->fill( $attributes );
        
    }

    public function list( $search = '', $page = 0, $sortedBy = ['s_id'], $sortedOrder = ['DESC'] )
    {
        $per_page = $this->per_page;
        $list = $this->setSelect()
                     ->addSearch( $search )
                     ->addOrderBy( $sortedBy, $sortedOrder )
                     ->addPaging( $page, $per_page )
                     ->addGroupBy( $this->table . '.id' )
                     ->queryBuilder->join('users', 'subscription.user_id', '=', 'users.id')
                     ->leftjoin('package', 'subscription.package_id', '=', 'package.id')
                     ->whereRaw('subscription_id <> ""')
                     ->select('users.*','subscription.id as s_id', 'subscription.created_at as created','subscription.status as s_status', 'subscription.*','package.*')
                     ->get();

        return $list;
    }

    public function alllist( $search = '', $page = 0, $sortedBy = ['id'], $sortedOrder = ['DESC'] )
    {
        $per_page = $this->per_page;
        $list = $this->setSelect()
                     ->addSearch( $search )
                     ->addOrderBy( $sortedBy, $sortedOrder )
                     ->addPaging( $page, $per_page )
                     ->addGroupBy( $this->table . '.id' )
                     ->get();

        return $list;
    }



    public function addGroupBy( $groupByName = null )
    {
        if ( ! empty( $groupByName ) ) {

            $this->queryBuilder->groupBy( $groupByName );
        }
        return $this;
    }

    public function addOrderBy( $sortedBy = [], $sortedOrder = [] )
    {

        if ( ! empty( $sortedBy ) && ! is_array( $sortedBy ) ) {
            $sortedBy = [$sortedBy];
        }

        if ( ! empty( $sortedOrder ) && ! is_array( $sortedOrder ) ) {
            $sortedOrder = [$sortedOrder];
        }

        if ( ! empty( $sortedBy ) && ! empty( $sortedOrder ) ) {

            foreach ( $sortedBy as $key => $sortedByName ) {

                $sortedOrder = ! empty( $sortedOrder[$key] ) ? $sortedOrder[$key] : $sortedOrder[0];

                $this->queryBuilder->orderBy( $sortedByName, $sortedOrder );
            }
        }
        return $this;
    }

    public function listTotalCount( $search = '', $sortedBy = ['subscription.id'], $sortedOrder = ['DESC'] )
    {
        $this->reset();
        $count = $this->setSelect()
                      ->addSearch( $search )
                      ->addOrderBy( $sortedBy, $sortedOrder )
                      ->addGroupBy( $this->table . '.id' )
                      ->queryBuilder->join('users', 'subscription.user_id', '=', 'users.id')
                      ->get()
                      ->count();

        return $count;
    }

    public function preparePagination( $total, $basePath )
    {
        $perpage = $this->per_page;
        $pageHelper = new \App\Classes\Helpers\PageHelper( $perpage, 'page' );
        $pageHelper->set_total( $total );
        $pageHelper->page_links( $basePath );
        return $pageHelper->page_links( $basePath );
    }

    public function saveRecord( $data )
    {
        
        if ( isset( $data['id'] ) && $data['id'] != '' ) 
        {
            $subscription = \App\Classes\Models\Subscription\Subscription::findOrFail( $data['id'] );
            $subscription->update( $data );
            $this->afterSave( $data, $subscription );
            $result['id'] = $subscription->id;
            $result['message'] = "正常に更新しました。";

        } else {
            $subscription = \App\Classes\Models\Subscription\Subscription::create( $data );
            $result['id'] = $subscription->id;
            $this->afterSave( $data, $subscription );
            $result['message'] = "お支払いが完了しました。";
        }

        $result['success'] = true;
        return $result;
    }

    public function removed( $id )
    {
        $this->beforeRemoved( $id );
        $deleteMemberObj = $this->display( $id );
        if ( ! empty( $deleteMemberObj ) ) {
            $delete = $deleteMemberObj->delete();
            $this->afterRemoved( $id );
            return $delete;
        }
        return false;
    }

    public function remove_by_userId( $id )
    {
        $this->setSelect()
            ->queryBuilder->where( 'user_id', $id )
            ->delete();
        return false;
    }

    public function display( $id )
    {
        return $this->load( $id );
    }

    public function load( $id )
    {
        $this->beforeLoad( $id );
        $return = $this->setSelect()
                       ->addSubscriptionIdFilter( $id )
                       ->get()
                       ->first();

        $this->afterLoad( $id, $return );
        return $return;
    }

    public function addSubscriptionIdFilter( $id = 0 )
    {
        $this->queryBuilder->where( 'id', $id );
        return $this;
    }

    public function addStatusFilter( $status = 1 )
    {
        
        $this->queryBuilder->where( 'status', $status );
        
        return $this;
    }

   

    public function updateRecord($data,$where_id)
    {
        
        $subscription = \App\Classes\Models\Subscription\Subscription::findOrFail($data['sub_primary_id']);
        
        $subscription->update( $data );
        
        $result['id'] = $subscription->id;
        return $result;
    }

    public function activeSubscription( $user_id, $status = 1 )
    {
        $return = $this->setSelect()
                       ->queryBuilder->where( 'user_id', $user_id )
                       ->leftjoin('package', 'subscription.package_id', '=', 'package.id')
                        ->select('subscription.id as s_id','package.created_at as p_created', 'subscription.*','package.*')
                       ->where( 'status', $status )
                       ->get()
                       ->first();

        
        return $return;
    }

}